package edu.georgetown.gucs.dictionary;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Pair;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;
import org.apache.commons.collections.CollectionUtils;

import java.util.*;

/**
 * @author Evan
 * @since 4/23/15.
 */
public class DictionaryTest {

    private Dictionary dictTest;

    private List<String> tokenizerNames;
    private String fileName;

    /**
     * Loads an XML Dictionary
     * * loads tokenizer names
     * * loads tokens with entry information
     *
     * @param dictionary to load from an XML file
     */
    public DictionaryTest(String dictionary) throws SdTextException {

        try{
        dictTest = new Dictionary(dictionary);

        fileName = dictionary;

        tokenizerNames = dictTest.getTokenizerNames();

        keys.addAll(dictTest.getDictionary());


    }catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    public Set<Pair<Token, Double>> getDictionary() {

        return keys;

    }

    public List<String> getTokenizers() {

        return tokenizerNames;

    }

    public void printTokens() {
        for (Pair<Token, Double> p : keys) {
            System.out.println(p.getA() + ":" + p.getB());
        }
    }

    Comparator<Pair<Token, Double>> comparator = new Comparator<Pair<Token, Double>>() {

        @Override
        public int compare(Pair<Token, Double> obj1, Pair<Token, Double> obj2) {

            int i = obj1.getA().toString().compareTo(obj2.getA().toString());
            if (i != 0) return i;

            return Double.compare(obj1.getB(), obj2.getB());

        }

    };
    SortedSet<Pair<Token, Double>> keys = new TreeSet<>(comparator);

    public Boolean findMismatch(Set<Pair<Token, Double>> incomingDictionary, String fileName) {
        //defensive copy
        Set<Pair<Token, Double>> baseCopy = new TreeSet<>(comparator);
        baseCopy.addAll(keys);


        Set<Pair<Token, Double>> incomingCopy = new TreeSet<>(comparator);
        incomingCopy.addAll(incomingDictionary);

        ArrayList comparison = (ArrayList) CollectionUtils.removeAll(keys, incomingCopy);


        System.out.println("[shared tokens] " + comparison + "\n");

        // remove shared tokens from each list
        incomingCopy.removeAll(comparison);
        baseCopy.removeAll(comparison);


        if (baseCopy.size() == 0 && incomingCopy.size() == 0 ) {

            return true;

        } else {

                System.out.println(fileName + " " + incomingCopy + "\n");
                System.out.println(this.fileName + " " + baseCopy + "\n");

            return false;
        }
    }

    public String getFileName(){ return fileName; }
}

