package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;

/**
 * Eliminates tokens that are shorter than a given length
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class MinimumLengthTokenizer extends Tokenizer {

    private static final long serialVersionUID = 8952335510230947262L;
    private final int minTokenLength;

    /**
     * Constructor that sets the minimum token length to be considered
     *
     * @param length the string value of the minimum token length to be included
     */
    public MinimumLengthTokenizer(String length) {
        this.minTokenLength = Integer.parseInt(length);
    }

    /**
     * Eliminates tokens that are shorter than the length specified in the constructor
     */
    @Override
    public List<Token> tokenize(List<Token> tokens) throws SdTextException {

        List<Token> tokenVector = Collections.synchronizedList(new Vector<Token>());
        try {
            synchronized (tokenVector) {
                for (Token token : tokens) {
                    if (token.name().length() >= this.minTokenLength) {
                        tokenVector.add(token);
                    }
                }
                return tokenVector;
            }
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return tokenVector;
    }
}
