package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;

/**
 * Separates tokens based on punctuation and removes punctuation from tokens
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 *
 */
public class StripPunctuationTokenizer extends Tokenizer {
    private static final long serialVersionUID = -5556452429806285951L;


    /**
     * Separates tokens based on punctuation and removes punctuation from tokens
     */
    @Override
    public List<Token> tokenize(List<Token> tokens) throws SdTextException {

        List<Token> tokenVector = Collections.synchronizedList(new Vector<Token>());
        try {
            synchronized (tokenVector) {
                for (Token token : tokens) {
                    // posix format for punctuation characters
                    String[] withoutPunctuation = token.name().split("\\p{Punct}", 0);
                    for (String name : withoutPunctuation) {
                        if (name.length() > 0) {
                            tokenVector.add(new Token(name, token.start_byte(), token.end_byte()));
                        }
                    }
                }
                return tokenVector;
            }
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return tokenVector;
    }
}
