package sdtext;

import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.dictionary.DictionaryTest;
import edu.georgetown.gucs.fingerprinter.FingerprintTest;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.*;
import enums.CommandLineHelp;
import java.nio.file.*;
import java.util.*;
import java.io.*;
import java.nio.charset.Charset;

/**
 * @author Evan
 * @since 3/24/15.
 */
public class ApiTest extends CLIOptions {

    private static final String TOKENIZER_CONFIG = "../doc/configuration_files/tokenizers_config.xml";	 
    
    // Instantiate the API to begin testing
    private API api = new API();


    
    public ApiTest(String[] args) throws SdTextException {

	    super(args);
    }

    /**
     * Reads a file that contains a list of fingerprint file names in order to compare them with text
     *
     * @param fileName the filename which contains a list of fingerprint files to compare to text
     * @return the fingerprint files that are named in filename
     */
    private Vector<String> readFileList(String fileName) {


        Path path = Paths.get(fileName);

        Vector<String> strings = new Vector<>();
	/*    try {
	      strings.addAll(Files.readAllLines(path.getFileName()));

	      } catch (IOException e) {
	      System.out.println("no files");
	      e.printStackTrace();
	      }
	*/
        return strings;
    }


    /**
     * calls CLIOptions in addition to allowing API access in order to conduct
     * command line testing
     */
    public void runTest() throws SdTextException {
	
	if (config.value == null) config.value = TOKENIZER_CONFIG;
	
	Boolean found = runProgram();

	if (!found){
	    switch (this.programName_) {

		//java -jar build/sdtext.jar Fingerprintstotext -i 4 -f fing.directory -m CosineSimilarityFingerprintMatcher -s 0
            case "FINGERPRINTSTOTEXT": {
	
		// Check that we have the required values, or print out error message
                if (dictionary.value == null ) {
		    System.err.println ("Missing dictionary");
		    printError(CommandLineHelp.FINGERPRINTSTOTEXT);
		}
		if ((source.value == null) && (input.value == null)) {
		    System.err.println ("Missing string for comparison");
		    printError(CommandLineHelp.FINGERPRINTSTOTEXT);
		}
		if ((directory.value == null) && (fingerprint.value == null)) {
		    System.err.println ("Missing fingerprint for comparison");
		    printError(CommandLineHelp.FINGERPRINTSTOTEXT);
		}
		

		// Figure out where the string to read is coming from
		if ((source.value != null) && (input.value != null)){
		    System.err.println("Cannot specify both the -g and -i options at the same time.");
		    printError(CommandLineHelp.FINGERPRINTSTOTEXT);
		}


		String contents = null;
		// Using a string from the command line for input
		if (source.value != null){
		    contents = source.value;
		}

		// Reading a document to get input strings
		if (input.value != null){
		    FileChecker.canRead(input.value,"APITest.Java","runTest");

		    Path path = Paths.get(input.value);

		    List<String> strings = new ArrayList<>();
		    try {
			strings.addAll(Files.readAllLines(path.getFileName(),
							  Charset.defaultCharset()));	    
			
			StringBuilder docContent = new StringBuilder();
			
			for (String str : strings) {
			    docContent.append(str);
			    docContent.append(" ");
			}
			
			contents = docContent.toString();
			
		    } catch (IOException e) {
			e.printStackTrace();
		    }
		}

		//System.err.println("DEBUG - content is: " + contents);
		
		
		// Figure out where the files to read are coming from
		if ((directory.value != null) && (fingerprint.value != null)){
		    System.err.println("Cannot specify both the -p and -f options at the same time.");
		    printError(CommandLineHelp.FINGERPRINTSTOTEXT);
		}

		// List of files is coming from a directory
		ArrayList<String> fingerprintFiles = new ArrayList<String>();
		if (directory.value != null){
		    FileChecker.canRead(directory.value,"APITest.Java","runTest");
		    FileLister fileFinder = new FileLister(directory.value);
		    for (File file : fileFinder.getFileList()){
			fingerprintFiles.add(file.getPath());
		    }
		}


		// List of files is coming from a file
		// May not handle spaces properly
		if (fingerprint.value != null){
		    FileChecker.canRead(fingerprint.value,"APITest.Java","runTest");

		    try {
			Scanner s = new Scanner(new File(fingerprint.value));
			while (s.hasNextLine()){
			    fingerprintFiles.add(s.nextLine());
			}
			s.close();
		    }
		    catch (FileNotFoundException e){
			System.err.println("File not found even though we checked");
		    }
   		}
		
		//System.err.println("DEBUG - got the following files: " + fingerprintFiles.toString());
				  

		
		// Set optional values to default if needed
                matcher.value = (matcher.value == null) ? "CosineSimilarityFingerprintMatcher" : matcher.value;		

		int minimumScore = 0;
		if (minScore.value > 0.0) {
		    minimumScore = (int) minScore.value;
		}
		
		// Can pass logging value directly
	                     
		Results results = api.cmpFingerprintFilesToText((int) minimumScore, // min match score
					  matcher.value, // Matcher name
					  contents, // String to compare
					  dictionary.value, // dictionary file name
					  fingerprintFiles, // vector of fingerprint file names
					  debug.value // logging level
	        		      );

		results.printResults();

                break;
            }
		//java -jar build/sdtext.jar CompareFingToDirectory -m CosineSimilarityFingerprintMatcher -s 0 -f fed4.fing -p ../sdtext/doc/input_files/federalist/
            case "COMPAREFINGTODIRECTORY": {

		
		if (dictionary.value == null ) {
		    System.err.println ("Missing dictionary");
		    printError(CommandLineHelp.COMPAREFINGTODIR);
		}

		if (fingerprint.value == null) {
		    System.err.println ("Missing fingerprint for comparison");
		    printError(CommandLineHelp.COMPAREFINGTODIR);
		}
		
                if (directory.value == null) {
		    System.err.println ("Missing directory");
                    printError(CommandLineHelp.COMPAREFINGTODIR);
                }

                matcher.value = (matcher.value == null) ? "CosineSimilarityFingerprintMatcher" : matcher.value;
                minScore.value = (minScore.value > 0) ? minScore.value : 0;

		Results results =  api.cmpFingerprintToDirectory((int) minScore.value,
								 matcher.value,
								 fingerprint.value,
								 directory.value,
								 dictionary.value,
								 debug.value);

                results.printResults();

                break;
            }
		// java -jar build/sdtext.jar CreateDictTrim -l 0 -h .6 -c ../configuration_files/tokenizers_config.xml -p ../enron -d enronV2.dict
            case "CREATEDICTTRIM": {

		//System.err.println("Inside create and trim");
		 if (dictionary.value == null){
		    System.err.println("Missing dictionary");
		    printError(CommandLineHelp.CREATEDICTTRIM);		    
		}

		if (minIDF.value < 0 || minIDF.value > 1 ||
		    maxIDF.value < 0 || maxIDF.value > 1){
		    System.err.println("Bad IDF value l:" + minIDF.value + " h:" + maxIDF.value);
		    printError(CommandLineHelp.CREATEDICTTRIM);		    
		}

		if (minIDF.value >= maxIDF.value) {
		    System.err.println("Bad IDF range l:" + minIDF.value + " h:" + maxIDF.value);
		    printError(CommandLineHelp.CREATEDICTTRIM);		    
                    
                }

                //dictionary is the output dictionary filename
                api.createDictAndTrim(minIDF.value, maxIDF.value, config.value, directory.value, dictionary.value, null);
		//		System.err.println("Returned");
                break;
            }
		//java -jar build/sdtext.jar BitVectorFingerprinter -i ../enron_9 -d enronV2.dict -o enronV2_9.fing -c ../configuration_files/fingerprinter_config.xml//
            case "BITVECTORFINGERPRINTER": {

                api.createFingerprint(fingerprint.value, dictionary.value, output.value, null );
                break;
            }

            case "TESTDICTIONARY": {

                if (dictionary.value == null || directory.value == null) {
                    printError(CommandLineHelp.TESTDICTIONARY);
                }

                DictionaryTest dict1 = new DictionaryTest(dictionary.value);
                DictionaryTest dict2 = new DictionaryTest(directory.value);

                // guard clauses
                // Compare tokenizer lists
                if (dict1.getTokenizers().removeAll(dict2.getTokenizers())) {

                    tokenizeTester(dict1.getTokenizers());

                }       // if dict1.getTokenizers is greater than zero there must be a mismatch
                if (dict1.getTokenizers().size() > 0 &&
		    dict2.getTokenizers().removeAll(dict1.getTokenizers())) {

                    tokenizeTester(dict2.getTokenizers());

                }

                // Compare dictionary
                if (dict2.findMismatch(dict1.getDictionary(), dict1.getFileName())) {
                    System.out.println("Dictionaries Match");
                }

                break;
            }

            case "TESTFINGERPRINT": {


                FingerprintTest fing1 = new FingerprintTest(source.value);
                FingerprintTest fing2 = new FingerprintTest(target.value);
		Dictionary dict = new Dictionary(dictionary.value);
		
                fing1.antlrBitSet(fing2.getFingerprint(), fing2.getDictionary());

                // assert (fingerprintComparison.size() == 2);
                // matchFingerprints(fingerprintComparison.get(0), fingerprintComparison.get(1));
                break;
            }

            default: {
                printHelp();
            }
	    }
	}
    }
    
    /*
     * Ancillary methods
     */

    public void matchFingerprints(Map<Integer, String> baseFingerprint,
                                  Map<Integer, String> incomingFingerprint) {

        if (baseFingerprint.size() != incomingFingerprint.size()) {

            System.out.println("Fingerprint sizes are different by " +
			       (baseFingerprint.size() - incomingFingerprint.size()));

        }

        for (Map.Entry<Integer, String> fingerprint : baseFingerprint.entrySet()) {

            if (incomingFingerprint.containsKey(fingerprint.getKey())) {

                if (incomingFingerprint.get(fingerprint.getKey()).compareTo(fingerprint.getValue()) != 0) {

                    System.out.println("Fingerprints have the same position,");
                    System.out.println("But different words in each place " + fingerprint.getValue());

                } else {

                    System.out.println("Fingerprints both contain: " + fingerprint.getValue());
                }

            } else {
                System.out.println("Fingerprint is missing: " + fingerprint.getValue());
            }

        }

    }

    /**
     * @param fileName the file for sdtext to read
     * @return the contents of the file that sdText reads
     */
    public String readDoc(String fileName) {

        Path path = Paths.get(fileName);

        List<String> strings = new ArrayList<>();
	try {
	    strings.addAll(Files.readAllLines(path.getFileName(),Charset.defaultCharset()));	    
	} catch (IOException e) {
	    e.printStackTrace();
	}

        return listToString(strings);
    }

    /**
     * @param strings converts a list of strings into a single string
     * @return a single string containing all of the lists contents
     */
    private String listToString(List<String> strings) {

        StringBuilder docContent = new StringBuilder();

        for (String str : strings) {
            docContent.append(str);
	    docContent.append(" ");
	}

        return docContent.toString();
    }

    /**
     * Prints tokenizers if there is a mismatch
     *
     * @param tokenizerList list of tokenizers used to create the dictionary
     */
    private void tokenizeTester(List<String> tokenizerList) {

        if (tokenizerList.size() > 0) {

            for (String tokenizer : tokenizerList) {
                System.out.println(tokenizer);
            }
        }

    }
}
